% LEMKEG Solves the extended vertical linear complementarity problem (EVLCP)
% An EVLCP(M,q) seeks a z that solves
%    min(w(1),w(2),...,w(p))=0
% where w(i)=M(i)z+q(i). 
% USAGE
%   [z,w,err]=lemkeg(M,q,z0);
% INPUTS
%   M  : np x n matrix (p nxn matrices stacked vertically)
%   q  : n x p matrix
%   z0 : n x 1 vector [optional starting value]
% OUTPUTS
%   z  : n x 1 vector (the solution if found)
%   w  : n x p matrix, the ith column is w(i)=M(i)z+q(i)
%   err : a scalar error flag:
%          0: Solution found
%          1: Maximum iterations exceeded
%          2: Unbounded ray termination
%          3: Initial basis cannot be computed - try new value of z0
%          4: Non-invertible matrix encountered
% If err is not requested, a warning message is displayed if err>0
%
% The input z0 provides an initial guess of the solution z.
% If z0 is omitted the origin is used as a starting guess.
%
% ALGORITHM
%   Uses a modified Lemke's almost complementary pivoting algorithm 

% Copyright (c) 2003, Paul L. Fackler
% paul_fackler@ncsu.edu

function [z,w,err,iter] = lemkeg(M,q,z0,attempts)
[n,p] = size(q);
np=n*p;
if nargin<3 | isempty(z0), z0=zeros(n,1); end  % default z0
if nargin<4, attempts=3; end                   % default attempts
if size(M)~=[np,n] | size(z0)~=[n 1]
  error('Inputs are not compatible'); 
end

piv_tol = 1e-8;
zer_tol = 1e-5;
maxiter = min(1000,50*n);
err     = 0;
iter    = 0;

z=z0;
w=reshape(M*z,n,p)+q;
[wmin,j]=min(w,[],2);
nonbas=sub2ind([n p],(1:n)',j);
bas=(1:np)';
bas(nonbas)=[];
try
  z=-(M(nonbas,:)\q(nonbas));
catch
  err=3; maxiter=0;            % initial basis matrix is singular
end

if err==0
  x=[q(bas)+M(bas,:)*z;z];
  % Determine initial leaving variable
  windex=1:np-n;                 % location of basic w variables
  zindex=np-n+1:np;              % location of basic z variables  
  [xmin,lvindex]=min(x(windex));
  if xmin>=0
    z=x(zindex); w=zeros(n,p); w(bas)=x(windex); return; % problem solved
  end
  leaving=bas(lvindex);

  % Get artificial variable
  U=x<0;                      % "covering vector"
  x=x-xmin*U;
  BU=M*U(np-n+1:end);
  BU=[U(1:np-n)-BU(bas,:);-BU(nonbas,:)];
  BU([bas;nonbas])=BU;
  % Swap artificial into position np-n
  bas(lvindex)=bas(np-n);
  bas(np-n)=0;
  x(lvindex)=x(np-n);
  x(np-n)=-xmin;
  % Set up B1 and B2
  B2=[BU';M'];
  B1=B2(:,bas(1:end-1));
  B2=B2(:,[nonbas;leaving]);

  artificialindex=np-n;   % used for restarting if ray termination occurs
  zminval=x(zindex);
end

oldwarn=warning;
warning off;

% Main iterations begin here
for iter=1:maxiter
  % Get new entering variable
  entindex=rem(leaving,n);
  if entindex==0, entindex=n; end
  entering=nonbas(entindex);
  nonbas(entindex)=leaving;
  d=zeros(1,n+1); d(entindex)=-1;
  d=d/B2; 
  if any(isnan(d) | isinf(d))
    if attempts>1                   % clear memory and try a new starting point
      clear z x w B1 B2 BU d bas nonbas theta j
      [z,w,err]=lemkeg(M,q,zminval,attempts-1);
      warning=oldwarn;
      return
    else
      err=4;
    end
    break
  end
  d=[d*B1 d]';  

  % Get new leaving variable
  j=find(d(windex)>piv_tol);        % indices of d>0 for w (never force z out)
  if isempty(j)                     % no new pivots - ray termination  
    if attempts>1                   % clear memory and try a new starting point
      clear z x w B1 B2 BU d bas nonbas theta j
      [z,w,err,iter2]=lemkeg(M,q,zminval,attempts-1);
      iter=iter+iter2;
      warning=oldwarn;
      return
    else
      err=2;
    end
    break
  end
  theta=min((x(j)+zer_tol)./d(j));    % minimal ratios, d>0
  j=j(find((x(j)./d(j))<=theta));
  if length(j)==1                     % only 1 candidate entering variable 
    lvindex=j;
  elseif any(j==np)                   % Always use artificial if possible
    lvindex=np;
  else                                % otherwise pick among set of max d 
    [theta,lvindex]=max(d(j)); 
    lvindex=find(d(j)==theta);
    if length(lvindex)>1              % degeneracy
      lvindex=ceil(length(lvindex)*rand(1,1));  % if multiple choose randomly
    end
    lvindex=j(lvindex);
  end
  leaving=bas(lvindex);
  bas(lvindex) = entering;

  % Check if done
  if leaving == 0
    z=-(M(nonbas,:)\q(nonbas));
    w=zeros(n,p); w(bas)=q(bas)+M(bas,:)*z;
    warning=oldwarn;
    return;
  end

  % Update x, B1 and B2
  ratio=x(lvindex)./d(lvindex);
  x = x - ratio*d;
  x(lvindex) = ratio;
  temp=B1(:,lvindex);
  B1(:,lvindex)=B2(:,entindex);
  B2(:,entindex)=B2(:,end);
  B2(:,end)=temp;

  % update restart information
  if xmin>x(artificialindex)
    xmin=x(artificialindex);
    zminval=x(zindex);
  end
end                                   % end of iterations

% Cleanup when unsuccessful
if iter>=maxiter,  err=1; end
z=x(zindex); 
w=reshape(M*z,n,p)+q; 

warning=oldwarn;
% Display warning messages if no error code is returned
if nargout<3 & err~=0
  s='Warning: solution not found - ';
  if err==1
    disp([s 'Iterations exceeded limit']);
  elseif err==2
    disp([s 'Unbounded ray']);
  elseif err==3
    disp([s 'Initial basis infeasible']);
  end
end

