% OSSOLVE Solves continuous time optimal switching models
% USAGE
%   [cv,snodes,x]=ossolve(model,fspace,snodes,cv);
% INPUTS
%   model          : a structure variable (defined below)
%   fspace         : an approximation function structure 
%   [snodes]       : nodal state values (n rows)
%   [cv]           : initial guess of solution coefficients (n by m)
% OUTPUTS
%   cv             : solution coefficients (n by m)
%   snodes         : nodal state values (n rows or cell array)
%   x              : optimal control values (n by m)
%   xindex         : regime boundary behavior variable for 1-D models
%
% The model structure should contain the following fields:
%   func     : problem definition file (see below)  
%   params   : additional parameters to pass to prob. def. file
%   C        : switching costs (m by m) or string 'variable'
%   L        : exogenous switch intensities (m by m) or string 'variable'
%   Q        : exogenous switching costs (m by m) or string 'variable'
%
% The problem definition file should have the format
%   function out=dscfile(flag,s,R,additional parameters)
%      switch flag
%        case 'f'
%          Return a matrix f representing the reward function
%        case 'mu'
%          Return a matrix mu representing the drift function
%              of the state transition equation
%        case 'sigma'
%          Return a matrix sigma representing the diffusion function
%              of the state transition equation
%        case 'rho'
%          Return a vector representing the (state contingent)
%              discount rates (or a scalar if discount rate is constant)
%      end
% For problems with d continuous states the sizes of these matrices
% should be as follows (with n=# of rows in s):
%            f     : n x 1
%            mu    : n x d
%            sigma : n x d x d (or n x d^2)
%            rho   : n x 1 (or 1 x 1)
% Optional flags when switch costs and intensities are non-constant:
%            C     : n x m
%            L     : n x m
%            Q     : n x m

function [cv,snodes,x,xindex]=dscsolve(model,fspace,snodes,cv)

alg = optget(mfilename,'alg','snewton');

  if nargin<3 | isempty(snodes), snodes=funnode(fspace); end
  if nargin<4, cv=[]; end
  d=fspace.d;  % dimension of the state 
  
  % Get basis matrices and nodal points
  bases=funbasx(fspace,snodes,[0;1;2]);
  Phi=ckron(bases.vals(1,d:-1:1));
  S=gridmake(snodes);
  n=prod(fspace.n);

% Unpack model variables
  func=model.func;                                                       % the name of the model function file
  if isfield(model,'params'); params=model.params; else params={}; end   % additional parameters to pass to func
  if isfield(model,'x'),  x=model.x;                                     % values of the regimes
  else,                   x=[];
  end
  % model.cost must be sepecified as 1) an mxm matrix 
  % or 2) a string variable (suggested as the word variable). If the latter, the
  % model function file must accept the flag 'C' and return an nxm matrix of
  % state dependent switching costs
  if ~isfield(model,'C')
    error('model.cost must be specified')
  elseif isnumeric(model.C)  
    C=model.C;
    m=size(C,1);
    Cvariable=logical(0);
    if isempty(x), x=(1:m)'; end
  else
    if isempty(x),
      temp=feval(func,'C',S,1,params{:});
      m=size(temp,2);
      x=(1:m)'; 
      C=zeros(n,m,m);
      C(:,:,1)=temp;
      temp=[];
    else 
      C=zeros(n,m,m);
      C(:,:,1)=feval(func,'C',S,x(1),params{:});
    end
    for i=2:m
      C(:,:,i)=feval(func,'C',S,x(i),params{:});
    end  
    Cvariable=logical(1);
  end
  nm=n*m;
  
  % Handle exogenous switching 
  %NOTE: variable intensities and costs not yet implemented!!
  if isfield(model,'L'), Lambda=model.L;
  else,                  Lambda=sparse(m,m);
  end

  if isfield(model,'Q'), Q=model.Q;
  else,                  Q=sparse(m,m);
  end
  
  % Check for resetting of states and preset value function
  if isfield(model,'reset'),  reset  = model.reset;  else reset  = []; end
  if isfield(model,'values'), values = model.values; else values = []; end
  
% Compute M and q
  M=sparse(0,0);
  q=zeros(nm,m);
  ee=speye(m);
  rho=feval(func,'rho',S,[],params{:});
  for i=1:m
    mu=feval(func,'mu',S,x(i,:),params{:});
    sigma=feval(func,'sigma',S,x(i,:),params{:});
    sumL=sum(Lambda(i,:));
    sumLQ=sum(Lambda(i,:).*Q(i,:));
    if isempty(sigma)
      Bi=ctbasemake(rho+sumL,bases,0)-ctbasemake(mu,bases,1);
    else
      Bi=ctbasemake(rho+sumL,bases,0)-ctbasemake(mu,bases,1)-ctbasemake(sigma,bases,2);
    end
    if sumL==0
      M=[M;(kron(ee(:,i)*ee(i,:),Bi)+kron(speye(m)-ones(m,1)*ee(i,:),Phi))];
    else
      M=[M;(kron(ee(:,i)*ee(i,:),Bi)...
            +kron(speye(m)-ones(m,1)*ee(i,:)-ee(:,i)*Lambda(i,:),Phi))];
    end
    fi=feval(func,'f',S,x(i,:),params{:});
    if Cvariable
      Ci=C(:,i,:);
    else
      Ci=C(:,i)';
      Ci=Ci(ones(n,1),:);
    end  
    if sumLQ==0
      q(:,i)=Ci(:)-kron(ee(:,i),fi);
    else
      q(:,i)=Ci(:)-kron(ee(:,i),fi-sumLQ);
    end
  end
  clear bases Ci C rho mu sigma fi
  
  % Alter any rows of value-matching at which resetting may occur
  if ~isempty(reset)
    for k=1:size(reset,1)
      i=reset(k,1); j=reset(k,2);
      Sk=snodes;
      for ii=1:d
        if ~isnan(reset(k,2+ii)), Sk{ii}=reset(k,2+ii); end; 
      end
      M((j-1)*nm+(i-1)*n+(1:n),(j-1)*n+(1:n))=-funbas(fspace,Sk);
    end
  end
  
  
  % Alter any rows at which value function is known
  if ~isempty(values)
    if size(values,2)~=d+2
      error('model.values has the wrong number of columns')
    end
    Svals=expandlist(values,{snodes,(1:m)'});
    Rvals=Svals(:,end-1);
    Vvals=-Svals(:,end);
    Svals(:,end-1:end)=[];
    [temp,ind]=nearest(S,Svals,1);
    ind=(Rvals-1)*n + ind;
    q(ind,:)=Vvals(:,ones(1,m));
    ind2=ind+(Rvals-1)*(nm*nm);
    [ind2,temp]=gridmake(ind2,(1:n)');
    ind2=ind2+(temp-1)*(nm*m);
    Phi=funbas(fspace,Svals);
    for j=1:m
      M(ind,:)=0;
      M(ind2)=Phi(:);
      ind=ind+nm;
      ind2=ind2+nm;
    end  
  end
  
  % Get starting values
  if isempty(cv) 
    ind=repmat((1:n)',1,m)+repmat((0:m-1)*(nm+n),n,1);
    cv=M(ind,:)\(-q(ind(:)));
  elseif size(cv,2)==1 
    cv=repmat(cv,1,m);  
  end

  % call the EVLCP solver - either lemkeg or snewton
  if upper(alg(1))=='L'
    [cv,w,err,iter]=lemkeg(M,q); 
  else
    [cv,w,err,iter]=snewton(M,q,cv(:)); 
  end
  cv=reshape(cv,n,m);
  if isempty(values)
    [res,xind]=min(w,[],2);
    x=reshape(xind,n,m);
  else
    [res,xind]=min(w,[],2);
    x=reshape(xind,n,m);
    waring('x may not be not correctly computed')
  end

% For 1-D problems get switchpoint information
% xindex(i,1-3) holds the approximate switchpoint, the regime switched to
% and the difference in the marginal values for the lower boundary
% of the no-switch region for regime i
% xindex(i,4-6) holds similar information for the upper boundary
  if size(S,2)>1
    xindex=[];
  else
    xindex=zeros(m,6);
    for i=1:m
      ii=find(x(:,i)==i);
      if ~isempty(ii)
        if ii(1)>1
          ss=S(ii(1)-1);
          xi=x(ii(1)-1,i);
          dval=funeval(cv(:,i),fspace,ss,1)-funeval(cv(:,xi),fspace,ss,1);
          xindex(i,1)=ss;
          xindex(i,2)=xi;
          xindex(i,3)=dval;
        else
          xindex(i,1)=fspace.a;
        end
        if ii(end)<n
          ss=S(ii(end)+1);
          xi=x(ii(end)+1,i);
          dval=funeval(cv(:,i),fspace,ss,1)-funeval(cv(:,xi),fspace,ss,1);
          xindex(i,4)=ss;
          xindex(i,5)=xi;
          xindex(i,6)=dval;
        else
          xindex(i,4)=fspace.b;
        end
      end
    end
  end
