% SNEWTON  Solves EVLCPs using Qi & Liao's Smoothing Newton Method
% USAGE
%   [z,w,err,it] = snewton(M,q,z0);
% INPUTS
%   M                      : nm x n matrix (m nxn matrices stacked vertically)
%   q                      : n x p matrix
%   z0                     : initial guess, n x 1 vector (optional)
%   tbar,gamma,sigma,delta : tuning parameters (optional)
%   tol                    : convergence tolerance (optional)
%   maxit                  : maximum number of iterations (optional)
%   maxsteps               : maximum number of backsteps (optional)
%   showiters              : display results of each iteration (optional)
% OUTPUTS
%   z   : solution vector
%   w   : nxm matrix of slackness variables: w(:,j) = M_jz+q(:,j)
%   err : maximal approximation error
%   it  : iterations required

% Ref.: SIAM J. Matrix Anal. Appl., 21, 45-66, 1999.

% Copyright (c) 2003, Paul L. Fackler
% paul_fackler@ncsu.edu

function [z,w,err,it]=snewton(M,q,z0,tbar,gamma,sigma,delta,tol,maxit,maxsteps,showiters)

[n,m]=size(q);
q(q>realmax/2)=realmax/2;  % avoid overflow and NaN problems 

% Default settings
if nargin<3 | isempty(z0), z = zeros(n,1); 
else, z=z0;   
end
if nargin<4  | isempty(tbar),      tbar      = 1;              end
if nargin<5  | isempty(gamma),     gamma     = 0.2;            end
if nargin<6  | isempty(sigma),     sigma     = 0.5e-4;         end
if nargin<7  | isempty(delta),     delta     = 0.5;            end
if nargin<8  | isempty(tol),       tol       = 1e-12;          end
if nargin<9  | isempty(maxit),     maxit     = min(2000,10*n); end
if nargin<10 | isempty(maxsteps),  maxsteps  = 25;             end
if nargin<11 | isempty(showiters), showiters = 0;              end 

wbar=[zeros(n,1);tbar];
t=tbar;
[fval,dm]=merit(M,q,z,t);
psi=fval'*fval;

W=psi;
minpsi=repmat(psi,1,5);
minpsi=[psi zeros(1,4)];
minpsii=1;
factor=2*sigma*(1-gamma*tbar);

for it=1:maxit
   % check if exact solution can be obtained
   if t<1e-5 
     ind=(x-1)*n+(1:n)';
     warning off
     zz=-(M(ind,:)\q(ind));
     warning on
     w=reshape(M*zz,n,m)+q;
     r=min(w,[],2);
     err=max(abs(r));
     if err<tol & ~any(isnan(zz(:)))
       z=zz;
       return
     end
   end
   % otherwise evaluate merit function
   [fval,dmz,dmt,x]=merit(M,q,z,t);
   psi=fval'*fval+t*t;
   % Check for singularity problems
   if isnan(psi) | isinf(psi), break; end
   % otherwise check if merit function value is small enough
   if psi<tol, break, end
   if showiters, fprintf('%4i %6.2e %6.2e\n',[it t psi]); end
  
   % update control parameters and determine Newton step
   beta=gamma*min(1,psi);
   checklevel=factor*psi;
   if psi>min(minpsi), W=psi; end
   minpsii=minpsii+1; if minpsii>5, minpsii=1; end
   minpsi(minpsii)=psi;
  
   % update iterates
   dt = beta*tbar-t;
   %ddmz=diag(dmz); dd=max(0,sum(abs(dmz))'-2*abs(ddmz)); 
   %dmz=dmz+sparse(1:n,1:n,ddmz-sign(ddmz).*dd,n,n);
   dz = dmz\(-fval-dmt*dt);

   for backstep=1:maxsteps 
      psinew = merit(M,q,z+dz,t+dt);
      psinew = psinew'*psinew+t*t;
      if psinew<=W-checklevel, 
         break, 
      end
      dz=dz*delta; dt=dt*delta; checklevel=checklevel*delta;
   end
   z = z+dz;
   t = t+dt;
end

if it>=maxit & nargout<4
 warning('Failure to converge in snewton');
end

w=reshape(M*z,n,m)+q;
r=min(w,[],2);
err=max(abs(r));

return


% merit function and its derivatives
function [phi,dphiz,dphit,x]=merit(M,q,z,t)
  [n,m]=size(q);
  w=reshape(M*z,n,m)+q;
  % the next line is a bottleneck - use MEX file mincol to speed it up
  % mw=min(w,[],2);  
  mw=mincol(w);
  ew=exp((mw(:,ones(1,m))-w)/t);
  sw=sum(ew,2);
  lsw=log(sw);
  phi=mw-t*lsw;

  if nargout>1
    temp=(1:n)';
    lambda=sparse(temp,temp,1./sw,n,n)*ew;
    dphiz=sparse(temp(:,ones(1,m)),1:n*m,lambda,n,n*m)*M;
    dphit=(mw-sum(w.*lambda,2))/t-lsw;
    [ignore,x]=max(lambda,[],2);
  end
